@@ -782,6 +782,135 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, regio
 		#logfile.close()
 	return 0
 
+import papermill as pm 
+import json
+import shutil
+import re
+from datetime import datetime
+
+def get_parameter_dict(nb_fname):
+    master_nb_parameters = {}
+    if os.path.isfile(nb_fname):
+        for kk, ii in pm.inspect_notebook(nb_fname).items():
+            #print(ii['default'])
+            value = ii['default']
+            inferred_type = ii['inferred_type_name']
+            if inferred_type == 'str':
+                real_value = value.replace('\'','')
+            elif inferred_type == 'bool':
+                if value == 'False':
+                    real_value = False
+                else:
+                    real_value = True
+                #real_value = gettype(inferred_type)(value)
+            else:
+                real_value = json.loads(value)
+            print(value, real_value)
+            master_nb_parameters.update({kk:real_value})
+    return master_nb_parameters
+
+def get_last_file(pattern="*/*.ipynb"):
+    # get list of files that matches pattern
+
+    files = list(filter(os.path.isfile, glob(pattern)))
+    files = list(filter(lambda x :('output' not in x), files))
+    # sort by modified time
+    files.sort(key=lambda x: os.path.getmtime(x))
+    if len(files)== 0:
+        return ''
+    # get last item in list
+    lastfile = files[-1]
+
+    print("Most recent file matching {}: {}".format(pattern,lastfile))
+    return lastfile
+
+def process_notebook():
+	help = sys.argv[0] + '\n'
+	help += '\n'
+	help += 'Process a notebook\n'
+
+	parser = argparse.ArgumentParser(description='Process Nustar Data',
+									 formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+	parser.add_argument('OBSID', metavar='obsid', type=str, nargs=1, help='OBSID with full path, e.g. Cen_X3/30101055002')
+	parser.add_argument("basenb", metavar='basenb', nargs=1, help="base notebook (absolute oath or relative to obsid)", type=str)
+
+	parser.add_argument("--force_matrix", help="Force computation of EN-Phase and T-Phase matrixes",
+						action='store_true')
+	parser.add_argument("--in_nb", help="Notebook to derive input parameters (to be found in OBSID)", type=str, default='none')
+	parser.add_argument("--in_yaml", help="yaml file with parameters (to be found in OBSID, or relative path)", type=str, default='none')
+	parser.add_argument("--select_nb", help="Select the last nb with pattern ep_??_*", action='store_true')
+
+	if len(sys.argv) == 1:
+		parser.print_help()
+		print(help)
+		sys.exit(1)
+
+	args = parser.parse_args()
+
+	obsid = args.OBSID[0]
+	if (not os.path.isdir(obsid)):
+		logger.error("The obsid location you specified does not exist "+obsid)
+		return 1
+
+	os.chdir(obsid)
+
+	if not os.path.isdir('obs_spec'):
+		print(obsid + ' has not been processed, return quitly')
+		return 0
+
+	print('We are in ' + os.getcwd())
+	base_nb = args.basenb[0]
+	if (not os.path.isfile(base_nb)):
+		logger.error("The base notebook you specified does not exist "+base_nb)
+		return 1
+
+	shutil.copy(base_nb, 'notebook_to_run.ipynb')
+
+	p  = re.compile('ep[_-][a-z]{2}[_-]*')
+
+	local_nb_parameters ={}
+
+	if args.select_nb:
+		my_nb = get_last_file('*.ipynb')
+		if p.match(my_nb):
+			print(f'Using {my_nb}')
+			local_nb_parameters.update(get_parameter_dict(my_nb))
+		else:
+			print(f'not using {my_nb}')
+
+	if args.in_nb != 'none':
+		if not os.path.isfile(args.in_nb):
+			print(f"{args.in_nb} does not exist, not using parameters")
+		else:
+			local_nb_parameters.update(get_parameter_dict(args.in_nb))
+	
+	if args.in_yaml != 'none':
+		if not os.path.isfile(args.in_yaml):
+			print(f"{args.in_yaml} does not exist, not using parameters")
+		else:
+			with open(args.in_yaml) as ff:
+				local_nb_parameters.update(get_parameter_dict(yaml.safe_load(ff)))
+
+	if args.force_matrix:
+		local_nb_parameters.update({'recompute_matrixes' : True})
+
+	now = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
+	
+	basename = os.path.basename(base_nb).replace('.ipynb','')
+	output_mosaics_notebook_path = basename+f'_output_{now}.ipynb'
+	print(f'Processing {obsid}')
+	print(base_nb + ' into ' + output_mosaics_notebook_path)
+	print(local_nb_parameters)
+	nb_mosaics_execution = pm.execute_notebook(
+			'notebook_to_run.ipynb',
+			output_mosaics_notebook_path,
+			parameters=local_nb_parameters,
+			log_output=True
+		)
+	os.remove('notebook_to_run.ipynb')
+
+	return nb_mosaics_execution
+	
 
 def process():
 	help = sys.argv[0] + '\n'
@@ -828,7 +957,6 @@ def process():
 
 	args = parser.parse_args()
 
-	#repository_location='/gpfs0/ferrigno/NuSTAR/Repository'
 	repository_location = args.repository
 	if (not os.path.isdir(repository_location)):
 		logger.error("The repository location you specified does not exist "+repository_location)
