@@ -1903,6 +1903,8 @@ def get_cross_correlation_with_error(pp, dpp, ee_pulsed=[], dee_pulsed=[], n_sim
         ax1.set_ylabel('Correlation')
         ax2.errorbar(ee_pulsed, lags, xerr=dee_pulsed, yerr=lag_errors, marker='x', linestyle='')
         ax2.set_ylabel('Lag (phase units)')
+        ax2.set_xlabel('Energy [keV]')
+        ax2.set_xscale('log')
         fig.savefig(stem+'_lag_correlation.png' )
 
     return lags, lag_errors, correlations, correlation_errors
